@@ -702,6 +702,16 @@ module Gen : sig
   val opt : ?ratio:float -> 'a t -> 'a option t
   (** [opt] is an alias of {!val:option} for backward compatibility. *)
 
+  val result : ?ratio:float -> 'a t -> 'e t -> ('a, 'e) result t
+  (** [result ~ratio okgen errgen] generates [Ok v] with [v] coming from [okgen]
+      or [Error e] with [e] coming from [errgen], depending on [ratio].
+
+      @param ratio a float between [0.] and [1.] indicating the probability of a sample to
+      be [Ok _] rather than [Error _].
+
+      @since 0.24 *)
+
+
   (** {3 Combining generators} *)
 
   val pair : 'a t -> 'b t -> ('a * 'b) t
@@ -729,20 +739,29 @@ module Gen : sig
   (** {4 Shrinks on [gen1], then [gen2], then ... } *)
 
   val tup2 : 'a t -> 'b t -> ('a * 'b) t
+  (** Combines two generators into a 2-tuple generator. *)
 
   val tup3 : 'a t -> 'b t -> 'c t -> ('a * 'b * 'c) t
+  (** Combines three generators into a 3-tuple generator. *)
 
   val tup4 : 'a t -> 'b t -> 'c t -> 'd t -> ('a * 'b * 'c * 'd) t
+  (** Combines four generators into a 4-tuple generator. *)
 
   val tup5 : 'a t -> 'b t -> 'c t -> 'd t -> 'e t -> ('a * 'b * 'c * 'd * 'e) t
+  (** Combines five generators into a 5-tuple generator. *)
 
   val tup6 : 'a t -> 'b t -> 'c t -> 'd t -> 'e t -> 'f t -> ('a * 'b * 'c * 'd * 'e * 'f) t
+  (** Combines six generators into a 6-tuple generator. *)
 
   val tup7 : 'a t -> 'b t -> 'c t -> 'd t -> 'e t -> 'f t -> 'g t -> ('a * 'b * 'c * 'd * 'e * 'f * 'g) t
+  (** Combines seven generators into a 7-tuple generator. *)
 
   val tup8 : 'a t -> 'b t -> 'c t -> 'd t -> 'e t -> 'f t -> 'g t -> 'h t -> ('a * 'b * 'c * 'd * 'e * 'f * 'g * 'h) t
+  (** Combines eight generators into an 8-tuple generator. *)
 
   val tup9 : 'a t -> 'b t -> 'c t -> 'd t -> 'e t -> 'f t -> 'g t -> 'h t -> 'i t -> ('a * 'b * 'c * 'd * 'e * 'f * 'g * 'h * 'i) t
+  (** Combines nine generators into a 9-tuple generator. *)
+
 
   (** {3 Convert a structure of generator to a generator of structure} *)
 
@@ -1111,7 +1130,15 @@ module Print : sig
   *)
 
   val int : int t
-  (** [int] is a printer of integer. *)
+  (** [int] is a printer of integers. *)
+
+  val int32 : int32 t
+  (** [int32] is a printer of 32-bit integers.
+      @since 0.24 *)
+
+  val int64 : int64 t
+  (** [int64] is a printer of 64-bit integers.
+      @since 0.24 *)
 
   val bool : bool t
   (** [bool] is a printer of boolean. *)
@@ -1132,6 +1159,12 @@ module Print : sig
   val option : 'a t -> 'a option t
   (** [option p] is a printer of ['a option], using [p] if it is a [Some]. *)
 
+  val result : 'a t -> 'e t -> ('a, 'e) result t
+  (** [result okp errp] is a printer of [('a,'e) result], using [okp] for printing [Ok _]
+      and [errp] for printing [Error _].
+
+      @since 0.24 *)
+
   val pair : 'a t -> 'b t -> ('a*'b) t
   (** [pair p1 p2] is a printer of pair. *)
 
@@ -1334,6 +1367,14 @@ module Observable : sig
   val int : int t
   (** [int] is an observable of [int]. *)
 
+  val int32 : int32 t
+  (** [int32] is an observable of [int32].
+      @since 0.24 *)
+
+  val int64 : int64 t
+  (** [int64] is an observable of [int64].
+      @since 0.24 *)
+
   val float : float t
   (** [float] is an observable of [float]. *)
 
@@ -1363,6 +1404,11 @@ module Observable : sig
   (** [option o] wraps the observable [o] of ['a] into an observable of
       ['a option]. *)
 
+  val result : 'a t -> 'e t -> ('a, 'e) result t
+  (** [result ok_o err_o] creates an [('a, 'e) result] observable out of
+      two observables [ok_o] and [err_o].
+      @since 0.24 *)
+
   val list : 'a t -> 'a list t
   (** [list o] wraps the observable [o] of ['a] into an observable of
       ['a list]. *)
@@ -1962,6 +2008,10 @@ val find_example_gen :
     QCheck2 is a major release and as such, there are (as few as possible)
     breaking changes, as well as functional changes you should be aware of.
 
+    The QCheck2 shrinkers haven't been battle tested as much as the
+    QCheck ones, and furthermore implement different reduction algorithms.
+    Please report if you encounter any issues porting tests from QCheck to QCheck2.
+
     {2 Minimal changes}
 
     Most of your QCheck (v1) code should be able to compile and run the first time you upgrade
